#ifndef __MATEWEATHER_ABOUT_H_
#define __MATEWEATHER_ABOUT_H_

/* $Id$ */

/*
 *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
 *
 *  This code released under the GNU GPL.
 *  Read the file COPYING for more information.
 *
 *  About box
 *
 */

#define MATEWEATHER_I_KNOW_THIS_IS_UNSTABLE

#include "mateweather.h"

G_BEGIN_DECLS

extern void mateweather_about_run (MateWeatherApplet *gw_applet);

G_END_DECLS

#endif /* __MATEWEATHER_ABOUT_H_ */

